<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Banco;

use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\LeitorRetorno;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Retorno;
use Financeiro\Entities\Boletreceber;
use Geral\Entities\Empresa;
use Geral\Entities\Cliente;

class LeitorRetornoSantander extends LeitorRetorno {

    const CODIGO_COMPENSACAO_SICOOB = "756";
    CONST ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    CONST ID_TP_ARQUIVO_HEADER_LOTE = '1';
    CONST ID_TP_ARQUIVO_SEGMENTO = '3';
    CONST ID_TP_ARQUIVO_TRAILER_LOTE = '5';
    CONST ID_TP_TRAILER_ARQUIVO = '9';
    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST RETORNO = 2;
    CONST SEGMENTO_A = 'A';
    CONST SEGMENTO_J = 'J';
    CONST SEGMENTO_O = 'O';
    CONST SEGMENTO_N = 'N';

    private $arquivo;
    private $cdagencia;
    private $cdconta;
    private $ocorrenciaHeader;
    private $contHeaderLote = 0;
    private $idocorr;

    /** @var $retorno Retorno */
    private $retorno;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco, $nrorg){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
            $this->nrorg = $nrorg;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getEstruturaArquivo(){
        return array(
            self::ID_TP_ARQUIVO_HEADER_ARQUIVO,
            self::ID_TP_ARQUIVO_HEADER_LOTE,
            self::ID_TP_ARQUIVO_SEGMENTO,
            self::ID_TP_ARQUIVO_TRAILER_LOTE,
            self::ID_TP_TRAILER_ARQUIVO
        );
    }

    public function leArquivo(){

        try {
            foreach ($this->arquivo as $linha) {
                $cont = $this->contlinha++;
                if ($linha){
                    $idsegmento = substr($linha, 13, 1);
                    if ($idsegmento == 'U')
                        $cont = $cont - 1;
                    else
                        $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha, 7, 1);
                    if(self::validaArquivo($linha, self::ID_TP_ARQUIVO_HEADER_ARQUIVO, self::getEstruturaArquivo(), $this->contlinha)) {
                        switch ($idRegistro) {

                            case(self::ID_TP_ARQUIVO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_SEGMENTO):
                                self::leDetalhe($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_TRAILER_LOTE):
                                self::leTrailerLote($linha);
                                break;

                            case(self::ID_TP_TRAILER_ARQUIVO):
                                break;
                            default:
                                throw new \Exception('Arquivo inválido.');
                        }
                    }
                    $this->retornos[$cont] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $this->nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * @param $cdempresa
     * @throws \Exception
     * @return
     */
    private function getIncricaoEmpresa($cdempresa){
        $empresa = Empresa::findOneByNrorgAndCdempresa($this->nrorg, $cdempresa);
        if($empresa)
            return $empresa->getNrinsjurempr();
        else
            throw new \Exception ('Empresa não Encontrada.');
    }


    /**
     * // =====================================================================  //
    // ====================  REGISTRO HEADER  DE ARQUIVO  ====================  //
    // =====================================================================  //
     * @param TXT $header
     */
    private function leHeaderArquivo($header){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header, 0, 3) != self::CODIGO_COMPENSACAO_SICOOB)
            self::gravaErros($this->contlinha, 'Banco não Cadastrado.', true);

        // CAMPO: Identificação do Arquivo de Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if (substr($header, 7, 1) == '0' && $this->contlinha != '1'){
            self::gravaErros($this->contlinha, 'Erro de identificação do registro header', true);
        }
    }

    /**
     * // =====================================================================  //
    // ====================  REGISTRO HEADER  DE LOTE  ====================  //
    // =====================================================================  //
     * @param TXT $header
     */
    private function leHeaderLote($header){
        // CAMPO: Tipo de Inscrição da empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $tpinscjur = trim(substr($header, 17, 1));
        if (empty($tpinscjur))
           self::gravaErros($this->contlinha, 'Tipo de inscrição jurídica não informada.', true);

        // CAMPO: Inscrição da empresa
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        $insjur = trim(substr($header, 18, 15));
        if (empty($insjur))
           self::gravaErros($this->contlinha, 'Inscrição jurídica não informada.', true);

        // CAMPO: Tipo De Registro
        // POSIÇÃO: 017 a 017
        // PICTURE: 9(1)
        $tiporegis = substr($header, 7, 1);
        if($tiporegis == '1' && $this->contlinha != '2')
            self::gravaErros($this->contlinha, 'Identificação do registro não é válida.', true);
    }

    /**
    // =====================================================================  //
    // ====================      REGISTRO DETALHE       ====================  //
    // =====================================================================  //
     * @param $detalhe
     */
    private function leDetalhe($detalhe){
        $segmento = substr($detalhe, 13, 1);
        if(substr($detalhe, 15, 2) != '28'){
            if($segmento == 'T'){
                $vrnominal = floatval(substr($detalhe, 77, 15));
                $idtitulobanco = trim(substr($detalhe, 40, 13));
                $cdfilial = trim(substr($detalhe, 100, 4));
                $nrlanbolrec = trim(substr($detalhe, 104, 10));
                $razaosocial = trim(substr($detalhe, 143, 40));
                $cdempresa = $this->cdempresa;
                $nossonumero = Boletreceber::getRepository();
                /** @var $nossonumero \Financeiro\Repositories\Boletreceber */
                $cdnosnumrec = $nossonumero->getNossoNumero($idtitulobanco, $cdempresa);
                $boleto = false;
                if(!empty($cdfilial) && !empty($nrlanbolrec))
                    /** @var $boleto Boletreceber */
                    $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $this->nrorg);
                elseif($cdnosnumrec)
                    /** @var $boleto Boletreceber */
                    $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                        $cdnosnumrec[0]['NRLANBOLREC'], $cdnosnumrec[0]['CDFILIAL'], $this->nrorg
                    );
                if(empty($idtitulobanco))
                    self::gravaErros($this->contlinha, 'Nosso número inválido.');
                $this->idocorr = trim(substr($detalhe, 15, 2));
                if(empty($this->idocorr)){
                    self::gravaErros($this->contlinha, 'Identificação da Ocorrência no Banco não informada.');
                }
                else{
                    $this->retorno->setOcorrencia(substr($detalhe, 15, 2));
                }
                $this->retorno->setCdlinha(str_pad($this->contlinha, 4, '0', STR_PAD_LEFT));
                $this->retorno->setBoleto('BOLETO');
                $this->retorno->setNossonumero($idtitulobanco);
                $this->retorno->setCarteira(trim(substr($detalhe, 53, 1)));
                $this->retorno->setCdagencia(substr($detalhe, 17, 4));
                $this->retorno->setCdconta(substr($detalhe, 22, 9));
                $this->retorno->setIdsitucnabr(self::situacaoCnab($this->idocorr));
                $this->retorno->setDtvencimento(substr($detalhe, 69, 8));
                $this->retorno->setVrnominal(round($vrnominal/100,2));
                $this->retorno->setErros(trim(substr($detalhe, 208, 10)));
                if(empty($razaosocial)){
                    if($boleto){
                        $cliente = Cliente::findOneByCdclienteAndNrorg($boleto->getCdcliente(), $this->nrorg);
                        $razaosocial = strtoupper($cliente->getNmrazsocclie());
                    }
                }
                $this->retorno->setNmrazsocclie($razaosocial);
                if($boleto){
                    $this->retorno->setVrfinalbrec($boleto->getVrfinalbrec());
                    $this->retorno->setVrbolrec($boleto->getVrbolrec());
                    $this->retorno->setIdstatrec($boleto->getIdstatbrec());
                    $this->retorno->setCdfilial($boleto->getCdfilial());
                    $this->retorno->setNrlanbolrec($boleto->getNrlanbolrec());
                    $this->retorno->setCdtpbaixarec($boleto->getCdtpbaixarec());
                    $this->retorno->setCdreducont($boleto->getCdreducont());
                }
                else{
                    $this->retorno->setNrlanbolrec($nrlanbolrec);
                }
            }
            else{
                $vrnominal = floatval(substr($detalhe, 77, 15));
                $this->retorno->setVrpago(round($vrnominal/100, 2));
                if($this->retorno->getIdsitucnabr() == '90')
                    $this->retorno->setDtpagamento(substr($detalhe, 145, 8));
                $vracrescimo = round(floatval(substr($detalhe, 17, 15)) / 100, 2) + round(floatval(substr($detalhe, 122, 15)) / 100, 2);
                $vrdesconto = round(floatval(substr($detalhe, 32, 15)) / 100, 2) + round(floatval(substr($detalhe, 47, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 62, 15)) / 100, 2) + round(floatval(substr($detalhe, 107, 15)) / 100, 2);
                $this->retorno->setVrdesconto($vrdesconto);
                $this->retorno->setVracrescimo($vracrescimo);
                $this->validaTransferenciaCarteira();
            }
        }
    }

    public function validaTransferenciaCarteira(){
        if ($this->idocorr != '04'){
            $idcredbanc = '10';
            if ($this->retorno->getCarteira() == '4'){
                $idcredbanc = '11';
            }
            $this->retorno->setIdNovaCarteira('000');
        }
        if (in_array($this->retorno->getCarteira(), ['1', '2', '3', '4'])){
            $this->retorno->setIdNovaCarteira($this->retorno->getCarteira());
        }
    }

    /**
     * @param $trailerlote
     */
    public function leTrailerLote($trailerlote){

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $ocorrenciastrailer = trim(substr($trailerlote,230,10));

        /** @var $retorno Retorno  */
        foreach($this->retornos as &$retorno){
            if($retorno->getIdheaderlote() == $this->contHeaderLote){
                //Adciona as ocorrencias do trailler e do header.
                $retorno->setOcorrencia($retorno->getOcorrencia().
                        $ocorrenciastrailer.
                        $this->ocorrenciaHeader);
            }
        }
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        if($cdocorrencia == '09')
            return '00';
        else if($cdocorrencia == '02' or $cdocorrencia == '19' or $cdocorrencia == '23')
            return '20';
        else if($cdocorrencia == '03' or $cdocorrencia == '26' or $cdocorrencia == '30')
            return '01';
        else if($cdocorrencia == '06' or $cdocorrencia == '17')
            return '90';
        return '10'; //Sem retorno
    }

}