<?php

namespace Teknisa\FIN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIN\Factories\RecalculoContabilizacaoTitulosBaixasFactory;
use Teknisa\FIN\Services\ContabilizacaoService;
use Teknisa\Libs\Util\Environment;
use DateTime;

class RecalculoContabilizacaoTitulosBaixasService {

    protected $factory;
    protected $contabilizacaoService;
    protected $environment;

    const LANC_INVALIDO_CODE = 1;
    const FORA_DO_PERIODO_CODE = 2;

    public function __construct(ContabilizacaoService $contabilizacaoService, EntityManager $entityManager, RecalculoContabilizacaoTitulosBaixasFactory $factory, Environment $environment) {
        $this->contabilizacaoService = $contabilizacaoService;
        $this->connection = $entityManager->getConnection();
        $this->factory = $factory;
        $this->environment = $environment;
    }


    public function findTitulosPagar (array $conditions) {
        $params = $this->buildParamsFromConditions($conditions);
        return $this->factory->findTitulosPagar($params);
    }

    public function findTitulosReceber (array $conditions) {
        $params = $this->buildParamsFromConditions($conditions);

        return $this->factory->findTitulosReceber($params);
    }

    public function findBaixasPagar (array $conditions) {
        $params = $this->buildParamsFromConditions($conditions);

        return $this->factory->findBaixasPagar($params);
    }

    public function findBaixasReceber (array $conditions) {
        $params = $this->buildParamsFromConditions($conditions);

        return $this->factory->findBaixasReceber($params);
    }

    protected function buildParamsFromConditions(array $conditions) {
        $params = array();
        foreach($conditions as $condition) {
            $params[$condition["columnName"]] = $condition["value"];
        }
        return $params;
    }

    public function validateRowTitulo($row) {

        $this->lastSelectContopectblResult = $this->factory->selectContopectbl($row['CDEMPRESA']);

        if ($this->lastSelectContopectblResult) {

          $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTINICCTBL']);

          $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTFINACCTBL']);

          $dtEntrSaid = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTENTRSAID']);

          if (!(($dtInicio <= $dtEntrSaid) && ($dtFinal >= $dtEntrSaid))) {
            throw new \Exception("Lançamento fora do período contábil", RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE);
          }

        } else {
          throw new \Exception("Lançamento inválido", RecalculoContabilizacaoTitulosBaixasService::LANC_INVALIDO_CODE);
        }

    }

    public function validateRowBaixaPagar($row) {

        $this->lastSelectContopectblResult = $this->factory->selectContopectbl($row['CDEMPRESA']);

        if ($this->lastSelectContopectblResult) {

          $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTINICCTBL']);

          $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTFINACCTBL']);

          $dtBaixaPag = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTBAIXAPAG']);

          if (!(($dtInicio <= $dtBaixaPag) && ($dtFinal >= $dtBaixaPag))) {
            throw new \Exception("Lançamento fora do período contábil", RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE);
          }

        } else {
          throw new \Exception("Lançamento inválido", RecalculoContabilizacaoTitulosBaixasService::LANC_INVALIDO_CODE);
        }

    }

    public function validateRowBaixaReceber($row) {

        $this->lastSelectContopectblResult = $this->factory->selectContopectbl($row['CDEMPRESA']);

        if ($this->lastSelectContopectblResult) {

          $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTINICCTBL']);

          $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTFINACCTBL']);

          $dtBaixaRec = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTBAIXAREC']);

          if (!(($dtInicio <= $dtBaixaRec) && ($dtFinal >= $dtBaixaRec))) {
            throw new \Exception("Lançamento fora do período contábil", RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE);
          }

        } else {
          throw new \Exception("Lançamento inválido", RecalculoContabilizacaoTitulosBaixasService::LANC_INVALIDO_CODE);
        }

    }

  public function getLastSelectContopectblResult() {

    if (isset($this->lastSelectContopectblResult)) {
      return $this->lastSelectContopectblResult;
    } else {
      return false;
    }

  }

  public function getRowsToBeUncheckedTitulos($rows) {

    $results = array();
    $rowsToBeUnchecked = array();

    foreach($rows as $row) {

      if (!isset($results[$row['CDEMPRESA']])) {
        $results[$row['CDEMPRESA']] = $this->factory->selectContopectbl($row['CDEMPRESA']);
      }

      if ($results[$row['CDEMPRESA']]) {

        $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTINICCTBL']);

        $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTFINACCTBL']);

        $dtEntrsaid = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTENTRSAID']);

        if (!(($dtInicio <= $dtEntrsaid) && ($dtFinal >= $dtEntrsaid))) {
          $rowsToBeUnchecked[] = $row;
        }
      } else {
        $rowsToBeUnchecked[] = $row;
      }

    }

    return $rowsToBeUnchecked;

  }

  public function getRowsToBeUncheckedBaixasPagar($rows) {

    $results = array();
    $rowsToBeUnchecked = array();

    foreach($rows as $row) {

      if (!isset($results[$row['CDEMPRESA']])) {
        $results[$row['CDEMPRESA']] = $this->factory->selectContopectbl($row['CDEMPRESA']);
      }
      if ($results[$row['CDEMPRESA']]) {

        $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTINICCTBL']);

        $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTFINACCTBL']);

        $dtBaixaPag = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTBAIXAPAG']);

        if (!(($dtInicio <= $dtBaixaPag) && ($dtFinal >= $dtBaixaPag))) {
          $rowsToBeUnchecked[] = $row;
        }
      } else {
        $rowsToBeUnchecked[] = $row;
      }

    }

    return $rowsToBeUnchecked;

  }

  public function getRowsToBeUncheckedBaixasReceber($rows) {

    $results = array();
    $rowsToBeUnchecked = array();

    foreach($rows as $row) {

      if (!isset($results[$row['CDEMPRESA']])) {
        $results[$row['CDEMPRESA']] = $this->factory->selectContopectbl($row['CDEMPRESA']);
      }
      if ($results[$row['CDEMPRESA']]) {

        $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTINICCTBL']);

        $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTFINACCTBL']);

        $dtBaixaRec = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTBAIXAREC']);

        if (!(($dtInicio <= $dtBaixaRec) && ($dtFinal >= $dtBaixaRec))) {
          $rowsToBeUnchecked[] = $row;
        }
      } else {
        $rowsToBeUnchecked[] = $row;
      }

    }

    return $rowsToBeUnchecked;

  }

  public function recontabilizarLancamentos($rows, $response) {

    $hadError = false;

    foreach($rows as $row) {
        $cdfilial = $row['CDFILIAL'];
        $cdempresa = $row['CDEMPRESA'];
        $nrlanctonf = $row['NRLANCTONF'];
        $nrparcelapag = $row['NRPARCELAPAG'];
        $nrseqvenpag = $row['NRSEQVENPAG'];
        $idstatpag = $row['IDSTATPAG'];
        $cdtpctpagar = $row['CDTPCTPAGAR'];
        $idutlanccpag = $row['IDUTLANCCPAG'];
        $this->connection->beginTransaction();
        try {
            $this->contabilizacaoService->contabilizaPagamento($cdfilial, $cdempresa, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $idstatpag, $cdtpctpagar, $idutlanccpag);
            $this->connection->commit();
        } catch (\Exception $e) {
            $hadError = true;
            $this->connection->rollback();
        }

    }

    if ($hadError) {
        $this->contabilizacaoService->saveContabilizacaoData($response);
        throw new \Exception("");
    }

  }

  public function recontabilizarRecebimentos($rows, $response) {

    $hadError = false;

    foreach($rows as $row) {
        $cdfilial = $row['CDFILIAL'];
        $cdempresa = $row['CDEMPRESA'];
        $nrlanctonf = $row['NRLANCTONF'];
        $nrparcelarec = $row['NRPARCELAREC'];
        $nrseqvenrec = $row['NRSEQVENREC'];
        $idstatrec = $row['IDSTATREC'];
        $cdtpctreceb = $row['CDTPCTRECEB'];
        $idutlanccrec = $row['IDUTLANCCREC'];
        $this->connection->beginTransaction();
        try {
            $this->contabilizacaoService->contabilizaRecebimento($cdfilial, $cdempresa, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $idstatrec, $cdtpctreceb, $idutlanccrec);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            $hadError = true;
        }

    }

    if ($hadError) {
        $this->contabilizacaoService->saveContabilizacaoData($response);
        throw new \Exception("");
    }

  }

  public function recontabilizarBaixasPagar($rows, $response) {

      $hadError = false;

      foreach($rows as $row) {
          $cdfilial = $row['CDFILIAL'];
          $cdempresa = $row['CDEMPRESA'];
          $nrbaixa = $row['NRBAIXAPAG'];
          $this->connection->beginTransaction();
          try {
              $this->contabilizacaoService->contabilizaBaixaPagar($cdempresa, $cdfilial, $nrbaixa);
              $this->connection->commit();
          } catch(\Exception $e) {
              $this->connection->rollback();
              $hadError = true;
          }

      }

      if ($hadError) {
          $this->contabilizacaoService->saveContabilizacaoData($response);
          throw new \Exception("");
      }

  }

  public function recontabilizarBaixasReceber($rows, $response) {

      $hadError = false;

      foreach($rows as $row) {
          $cdempresa = $row['CDEMPRESA'];
          $cdfilial = $row['CDFILIAL'];
          $cdfiliproc = $this->environment->getCdFilial();
          $nrbaixa = $row['NRBAIXAREC'];
          $this->connection->beginTransaction();
          try {
              $this->contabilizacaoService->contabilizaBaixaReceber($cdempresa, $cdfilial, $cdfiliproc, $nrbaixa);
              $this->connection->commit();
          } catch(\Exception $e) {
              $this->connection->rollback();
              $hadError = true;
          }

      }

      if ($hadError) {
          $this->contabilizacaoService->saveContabilizacaoData($response);
          throw new \Exception("");
      }

  }


}

